@@ -7,14 +7,7 @@ is based on Suds and extends it with ICAT specific features.
 .. _ICAT: https://icatproject.org/
 """
 
-import sys
-import warnings
-
-if sys.version_info < (3, 4):
-    warnings.warn("Support for Python versions older then 3.4 is deprecated  "
-                  "and will be removed in python-icat 1.0", DeprecationWarning)
-
-__version__ = "0.20.0"
+__version__ = "1.0.0"
 
 #
 # Default import
