@@ -1,14 +1,8 @@
 """Exception handling.
 """
 
-import sys
+from collections.abc import Mapping
 import warnings
-try:
-    # Python 3.3 and newer
-    from collections.abc import Mapping
-except ImportError:
-    # Python 2
-    from collections import Mapping
 import suds
 
 __all__ = [
@@ -33,8 +27,6 @@ __all__ = [
     'SearchAssertionError', 'DataConsistencyError', 
     # icat.ids
     'IDSResponseError', 
-    # icat.icatcheck
-    'GenealogyError',
     ]
 
 
@@ -55,25 +47,11 @@ class _BaseException(Exception):
 
     """
     def __init__(self, *args):
-        super(_BaseException, self).__init__(*args)
+        super().__init__(*args)
         if hasattr(self, '__cause__'):
             self.__cause__ = None
 
 
-def stripCause(e):
-    """Try to suppress misleading context from an exception.
-
-    .. deprecated:: 0.14.0
-       Not needed any more, embedded in
-       :exc:`icat.exception._BaseException` now.
-    """
-    warnings.warn("stripCause() is deprecated and will be removed "
-                  "in python-icat 1.0.", DeprecationWarning, 2)
-    if hasattr(e, '__cause__'):
-        e.__cause__ = None
-    return e
-
-
 # ========== Exceptions thrown by the ICAT or IDS server ===========
 
 class ServerError(_BaseException):
@@ -89,10 +67,10 @@ class ServerError(_BaseException):
         """
         if isinstance(error, suds.WebFault):
             try:
-                message = self._convertmsg(error.fault.faultstring)
+                message = str(error.fault.faultstring)
             except AttributeError:
-                message = self._convertmsg(str(error))
-            super(ServerError, self).__init__(message)
+                message = str(error)
+            super().__init__(message)
             self.status = status
             self.message = message
             self.fault = error.fault
@@ -108,18 +86,18 @@ class ServerError(_BaseException):
         elif isinstance(error, Mapping):
             # Deliberately not fetching KeyError here.  Require the
             # field to be present.  Only 'offset' is optional.
-            message = self._convertmsg(error['message'])
-            super(ServerError, self).__init__(message)
+            message = str(error['message'])
+            super().__init__(message)
             self.status = status
             self.message = message
             self.type = str(error['code'])
             self.offset = error.get('offset', None)
-        elif isinstance(error, basestring):
+        elif isinstance(error, str):
             # For compatibility with other exception classes, also
             # allow the constructor to be called with just a message
             # string as argument.
-            message = self._convertmsg(error)
-            super(ServerError, self).__init__(message)
+            message = str(error)
+            super().__init__(message)
             self.status = None
             self.message = message
             self.type = None
@@ -135,17 +113,6 @@ class ServerError(_BaseException):
             if self.offset < 0:
                 self.offset = None
 
-    def _convertmsg(self, msg):
-        # msg may be a str, a suds.sax.text.Text instance, or (only
-        # for Python 2) an unicode instance.  In Python 2, we must
-        # convert it to a pure ascii string for Exception.  In Python
-        # 3, we convert it to a str.  It may still contain non-ascii
-        # chars, but this is ok for Exception in Python 3.
-        if sys.version_info < (3, 0):
-            return msg.encode('ascii', 'replace')
-        else:
-            return str(msg)
-
 
 class ICATError(ServerError):
     """Base class for the errors raised by the ICAT server.
@@ -321,7 +288,7 @@ class QueryNullableOrderWarning(QueryWarning):
     def __init__(self, attr):
         msg = ("ordering on a nullable many to one relation implicitly "
                "adds a '%s IS NOT NULL' condition." % attr)
-        super(QueryNullableOrderWarning, self).__init__(msg)
+        super().__init__(msg)
 
 class QueryOneToManyOrderWarning(QueryWarning):
     """Warn about using a one to many relation for ordering.
@@ -331,7 +298,7 @@ class QueryOneToManyOrderWarning(QueryWarning):
     def __init__(self, attr):
         msg = ("ordering on a one to many relation %s may surprisingly "
                "affect the search result." % attr)
-        super(QueryOneToManyOrderWarning, self).__init__(msg)
+        super().__init__(msg)
 
 
 # ======== Exceptions raised in icat.client and icat.entity ========
@@ -351,7 +318,7 @@ class ClientVersionWarning(Warning):
         else:
             msg = ("%s is not supported (%s), "
                    "expect problems and weird behavior!" % (icatstr, comment))
-        super(ClientVersionWarning, self).__init__(msg)
+        super().__init__(msg)
 
 class ICATDeprecationWarning(DeprecationWarning):
     """Warn about using an API feature that may get removed in future ICAT
@@ -364,7 +331,7 @@ class ICATDeprecationWarning(DeprecationWarning):
             icatstr = "ICAT version %s" % version
         msg = ("%s has been deprecated and is expected to get removed in %s." 
                % (feature, icatstr))
-        super(ICATDeprecationWarning, self).__init__(msg)
+        super().__init__(msg)
 
 class EntityTypeError(_BaseException, TypeError):
     """An invalid entity type has been used.
@@ -384,7 +351,7 @@ class VersionMethodError(_BaseException):
         else:
             icatstr = "%s version %s" % (service, version)
         msg = ("%s is not supported in %s." % (method, icatstr))
-        super(VersionMethodError, self).__init__(msg)
+        super().__init__(msg)
 
 class SearchResultError(_BaseException):
     """A search result does not conform to what should have been expected.
@@ -415,7 +382,7 @@ class SearchAssertionError(SearchResultError):
             msg = ('Number of objects found (%d) is not within '
                    'the expected bounds between %d and %d on query: "%s"'
                    % (num, assertmin, assertmax, query))
-        super(SearchAssertionError, self).__init__(msg)
+        super().__init__(msg)
         self.query = query
         self.assertmin = assertmin
         self.assertmax = assertmax
@@ -432,15 +399,3 @@ class IDSResponseError(_BaseException):
     """The response from the IDS was not what should have been expected.
     """
     pass
-
-
-# ============== Exceptions raised in icat.icatcheck ===============
-
-class GenealogyError(_BaseException):
-    """Error in the genealogy of entity types.
-
-    .. deprecated:: 0.17
-       Only used in :mod:`icat.icatcheck` which in turn is deprecated.
-    """
-    pass
-
