@@ -1,17 +1,14 @@
 """Test the icat.query module.
 """
 
-from __future__ import print_function
 import datetime
-from distutils.version import StrictVersion as Version
 import re
-import sys
 import warnings
 import pytest
 import icat
 import icat.config
 from icat.query import Query
-from conftest import getConfig, require_icat_version, UtcTimezone
+from conftest import getConfig, icat_version, require_icat_version, UtcTimezone
 
 
 @pytest.fixture(scope="module")
@@ -24,12 +21,23 @@ def client(setupicat):
 # Note: the number of objects returned in the queries and their
 # attributes obviously depend on the content of the ICAT and need to
 # be kept in sync with the reference input used in the setupicat
-# fixture.
-
+# fixture.  This content also depends on the version of ICAT server we
+# are talking to and the ICAT schema this server provides.
+#
+# Note: the exact query string is considered an implementation detail
+# that is deliberately not tested here.  We limit the tests to check
+# the presence of some key fragments in the various clauses of the
+# queries.
 
 investigation = None
 tzinfo = UtcTimezone() if UtcTimezone else None
 
+# The the actual number of rules in the test data differs with the
+# ICAT version.
+have_icat_5 = 0 if icat_version < "5.0" else 1
+all_rules = 110 + 48*have_icat_5
+grp_rules = 50 + 30*have_icat_5
+
 @pytest.mark.dependency(name='get_investigation')
 def test_query_simple(client):
     """A simple query for an investigation by name.
@@ -39,6 +47,12 @@ def test_query_simple(client):
     name = "10100601-ST"
     query = Query(client, "Investigation", conditions={"name":"= '%s'" % name})
     print(str(query))
+    assert "Investigation" in query.select_clause
+    assert query.join_clause is None
+    assert "name" in query.where_clause
+    assert query.order_clause is None
+    assert query.include_clause is None
+    assert query.limit_clause is None
     res = client.search(query)
     assert len(res) == 1
     investigation = res[0]
@@ -60,6 +74,12 @@ def test_query_datafile(client):
     }
     query = Query(client, "Datafile", conditions=conditions)
     print(str(query))
+    assert "Datafile" in query.select_clause
+    assert "investigation" in query.join_clause
+    assert dfdata['investigation'] in query.where_clause
+    assert query.order_clause is None
+    assert query.include_clause is None
+    assert query.limit_clause is None
     qstr = str(query)
     res = client.search(query)
     assert len(res) == 1
@@ -95,6 +115,10 @@ def test_query_investigation_includes(client):
                   conditions={"id": "= %d" % investigation.id},
                   includes=includes)
     print(str(query))
+    assert "Investigation" in query.select_clause
+    assert query.join_clause is None
+    assert "id" in query.where_clause
+    assert "investigationInstruments" in query.include_clause
     res = client.search(query)
     assert len(res) == 1
     inv = res[0]
@@ -117,6 +141,11 @@ def test_query_instruments(client):
                                "= %d" % investigation.id },
                   includes={"facility", "instrumentScientists.user"})
     print(str(query))
+    assert "Instrument" in query.select_clause
+    assert "investigation" in query.join_clause
+    assert "id" in query.where_clause
+    assert "name" in query.order_clause
+    assert "instrumentScientists" in query.include_clause
     res = client.search(query)
     assert len(res) == 1
     instr = res[0]
@@ -133,6 +162,10 @@ def test_query_datafile_by_investigation(client):
                   includes={"dataset", "datafileFormat.facility",
                             "parameters.type.facility"})
     print(str(query))
+    assert "Datafile" in query.select_clause
+    assert "investigation" in query.join_clause
+    assert "id" in query.where_clause
+    assert "datafileFormat" in query.include_clause
     res = client.search(query)
     assert len(res) == 4
 
@@ -142,6 +175,11 @@ def test_query_relateddatafile(client):
     """
     query = Query(client, "RelatedDatafile", order=True)
     print(str(query))
+    assert "RelatedDatafile" in query.select_clause
+    assert query.where_clause is None
+    assert query.order_clause is not None
+    assert query.include_clause is None
+    assert query.limit_clause is None
     res = client.search(query)
     assert len(res) == 1
 
@@ -150,9 +188,14 @@ def test_query_datacollection(client):
     """
     query = Query(client, "DataCollection", order=True)
     print(str(query))
-    assert "id" in query.order
+    assert "DataCollection" in query.select_clause
+    assert query.join_clause is None
+    assert query.where_clause is None
+    assert "id" in query.order_clause
+    assert query.include_clause is None
+    assert query.limit_clause is None
     res = client.search(query)
-    assert len(res) == 2
+    assert len(res) == 3 + 2*have_icat_5
 
 def test_query_datafiles_datafileformat(client, recwarn):
     """Datafiles ordered by format.
@@ -165,8 +208,14 @@ def test_query_datafiles_datafileformat(client, recwarn):
     assert issubclass(w.category, icat.QueryNullableOrderWarning)
     assert "datafileFormat" in str(w.message)
     print(str(query))
+    assert "Datafile" in query.select_clause
+    assert "datafileFormat" in query.join_clause
+    assert query.where_clause is None
+    assert query.order_clause is not None
+    assert query.include_clause is None
+    assert query.limit_clause is None
     res = client.search(query)
-    assert len(res) == 10
+    assert len(res) == 10 + have_icat_5
 
 @pytest.mark.dependency(depends=['get_investigation'])
 def test_query_order_direction(client):
@@ -180,6 +229,10 @@ def test_query_order_direction(client):
                   conditions={ "dataset.investigation.id":
                                "= %d" % investigation.id })
     print(str(query))
+    assert "Datafile" in query.select_clause
+    assert "investigation" in query.join_clause
+    assert "id" in query.where_clause
+    assert "name" in query.order_clause
     res = client.search(query)
     assert len(res) == 4
     # Ascending order is the default, so we should get the same result:
@@ -218,6 +271,10 @@ def test_query_order_direction_relation(client):
     # in ascending order:
     query = Query(client, "Datafile", order=[("dataset", "DESC"), "name"])
     print(str(query))
+    assert "Datafile" in query.select_clause
+    assert "dataset" in query.join_clause
+    assert query.where_clause is None
+    assert "name" in query.order_clause
     dff = client.search(query)
     # verify:
     offs = 0
@@ -234,8 +291,11 @@ def test_query_condition_greaterthen(client):
     condition = {"datafileCreateTime": ">= '2012-01-01'"}
     query = Query(client, "Datafile", conditions=condition)
     print(str(query))
+    assert "Datafile" in query.select_clause
+    assert query.join_clause is None
+    assert "datafileCreateTime" in query.where_clause
     res = client.search(query)
-    assert len(res) == 4
+    assert len(res) == 4 + have_icat_5
     condition = {"datafileCreateTime": "< '2012-01-01'"}
     query = Query(client, "Datafile", conditions=condition)
     print(str(query))
@@ -248,9 +308,12 @@ def test_query_condition_list(client):
     condition = {"datafileCreateTime": [">= '2012-01-01'", "< '2013-01-01'"]}
     query = Query(client, "Datafile", conditions=condition)
     print(str(query))
+    assert "Datafile" in query.select_clause
+    assert query.join_clause is None
+    assert "datafileCreateTime" in query.where_clause
     qstr = str(query)
     res = client.search(query)
-    assert len(res) == 3
+    assert len(res) == 3 + have_icat_5
 
     # The last example also works by adding the conditions separately.
     query = Query(client, "Datafile")
@@ -259,16 +322,19 @@ def test_query_condition_list(client):
     print(str(query))
     assert str(query) == qstr
     res = client.search(query)
-    assert len(res) == 3
+    assert len(res) == 3 + have_icat_5
 
 @pytest.mark.dependency(depends=['get_investigation'])
 def test_query_in_operator(client):
-    """Using "id in (i)" rather then "id = i" also works.
+    """Using "id in (i)" rather than "id = i" also works.
     (This may be needed to work around ICAT Issue 128.)
     """
     query = Query(client, "Investigation",
                   conditions={"id": "in (%d)" % investigation.id})
     print(str(query))
+    assert "Investigation" in query.select_clause
+    assert query.join_clause is None
+    assert "in" in query.where_clause
     res = client.search(query)
     assert len(res) == 1
     inv = res[0]
@@ -282,8 +348,9 @@ def test_query_condition_obj(client):
     """
     query = Query(client, "Rule", conditions={"grouping": "IS NULL"})
     print(str(query))
+    assert "Rule" in query.select_clause
     res = client.search(query)
-    assert len(res) == 60
+    assert len(res) == all_rules - grp_rules
 
 def test_query_condition_jpql_function(client):
     """Functions may be applied to field names of conditions.
@@ -296,6 +363,9 @@ def test_query_condition_jpql_function(client):
     }
     query = Query(client, "Investigation", conditions=conditions)
     print(str(query))
+    assert "Investigation" in query.select_clause
+    assert "datasets" in query.join_clause
+    assert "UPPER" in query.where_clause
     res = client.search(query)
     assert len(res) == 1
 
@@ -305,9 +375,13 @@ def test_query_condition_jpql_function_namelen(client):
     of the JPQL function in the condition is easier to verify in the
     result.
     """
-    conditions = { "LENGTH(fullName)": "> 11" }
+    conditions = { "name": "LIKE 'db/%'",
+                   "LENGTH(fullName)": "> 11" }
     query = Query(client, "User", conditions=conditions)
     print(str(query))
+    assert "User" in query.select_clause
+    assert query.join_clause is None
+    assert "LENGTH" in query.where_clause
     res = client.search(query)
     assert len(res) == 4
 
@@ -316,9 +390,13 @@ def test_query_condition_jpql_function_mixed(client):
     This test case failed for an early implementation of JPQL
     functions, see discussion in #89.
     """
-    conditions = { "LENGTH(fullName)": "> 11", "fullName": "> 'C'" }
+    conditions = { "name": "LIKE 'db/%'",
+                   "LENGTH(fullName)": "> 11", "fullName": "> 'C'" }
     query = Query(client, "User", conditions=conditions)
     print(str(query))
+    assert "User" in query.select_clause
+    assert query.join_clause is None
+    assert "LENGTH" in query.where_clause
     res = client.search(query)
     assert len(res) == 3
 
@@ -329,9 +407,13 @@ def test_query_order_jpql_function(client):
     fullName.  (In the example data, the longest and second longest
     fullName is somewhat ambiguous due to character encoding issues.)
     """
-    query = Query(client, "User",
+    query = Query(client, "User", conditions={ "name": "LIKE 'db/%'" },
                   order=[("LENGTH(fullName)", "DESC")], limit=(2,1))
     print(str(query))
+    assert "User" in query.select_clause
+    assert query.join_clause is None
+    assert "LENGTH" in query.order_clause
+    assert query.limit_clause is not None
     res = client.search(query)
     assert len(res) == 1
     assert res[0].fullName == "Nicolas Bourbaki"
@@ -341,9 +423,12 @@ def test_query_rule_order(client):
     """
     query = Query(client, "Rule", order=True)
     print(str(query))
-    assert "id" in query.order
+    assert "Rule" in query.select_clause
+    assert query.join_clause is None
+    assert query.where_clause is None
+    assert "id" in query.order_clause
     res = client.search(query)
-    assert len(res) == 104
+    assert len(res) == all_rules
 
 def test_query_rule_order_group(client, recwarn):
     """Ordering rule on grouping implicitely adds a "grouping IS NOT NULL"
@@ -357,8 +442,12 @@ def test_query_rule_order_group(client, recwarn):
     assert issubclass(w.category, icat.QueryNullableOrderWarning)
     assert "grouping" in str(w.message)
     print(str(query))
+    assert "Rule" in query.select_clause
+    assert "grouping" in query.join_clause
+    assert query.where_clause is None
+    assert "what" in query.order_clause
     res = client.search(query)
-    assert len(res) == 44
+    assert len(res) == grp_rules
 
 def test_query_rule_order_group_suppress_warn_cond(client, recwarn):
     """The warning can be suppressed by making the condition explicit.
@@ -368,8 +457,12 @@ def test_query_rule_order_group_suppress_warn_cond(client, recwarn):
                   conditions={"grouping": "IS NOT NULL"})
     assert len(recwarn.list) == 0
     print(str(query))
+    assert "Rule" in query.select_clause
+    assert "grouping" in query.join_clause
+    assert "grouping" in query.where_clause
+    assert "what" in query.order_clause
     res = client.search(query)
-    assert len(res) == 44
+    assert len(res) == grp_rules
 
 def test_query_rule_order_group_suppress_warn_join(client, recwarn):
     """Another option to suppress the warning is to override the JOIN spec.
@@ -381,8 +474,12 @@ def test_query_rule_order_group_suppress_warn_join(client, recwarn):
                   join_specs={"grouping": "INNER JOIN"})
     assert len(recwarn.list) == 0
     print(str(query))
+    assert "Rule" in query.select_clause
+    assert "INNER JOIN" in query.join_clause
+    assert query.where_clause is None
+    assert "what" in query.order_clause
     res = client.search(query)
-    assert len(res) == 44
+    assert len(res) == grp_rules
 
 def test_query_rule_order_group_left_join(client, recwarn):
     """Another option to suppress the warning is to override the JOIN spec.
@@ -393,11 +490,15 @@ def test_query_rule_order_group_left_join(client, recwarn):
                   join_specs={"grouping": "LEFT OUTER JOIN"})
     assert len(recwarn.list) == 0
     print(str(query))
+    assert "Rule" in query.select_clause
+    assert "LEFT OUTER JOIN" in query.join_clause
+    assert query.where_clause is None
+    assert "what" in query.order_clause
     res = client.search(query)
-    assert len(res) == 104
+    assert len(res) == all_rules
 
 def test_query_order_one_to_many(client, recwarn):
-    """Sort on a related object in a one yo many relation.
+    """Sort on a related object in a one to many relation.
     This has been enabled in #84, but a warning is still emitted.
     """
     recwarn.clear()
@@ -407,6 +508,10 @@ def test_query_order_one_to_many(client, recwarn):
     assert issubclass(w.category, icat.QueryOneToManyOrderWarning)
     assert "investigationInstruments" in str(w.message)
     print(str(query))
+    assert "Investigation" in query.select_clause
+    assert "instrument" in query.join_clause
+    assert query.where_clause is None
+    assert "fullName" in query.order_clause
     res = client.search(query)
     assert len(res) == 3
 
@@ -419,11 +524,16 @@ def test_query_order_one_to_many_warning_suppressed(client, recwarn):
                   join_specs={"investigationInstruments": "INNER JOIN"})
     assert len(recwarn.list) == 0
     print(str(query))
+    assert "Investigation" in query.select_clause
+    assert "INNER JOIN" in query.join_clause
+    assert "instrument" in query.join_clause
+    assert query.where_clause is None
+    assert "fullName" in query.order_clause
     res = client.search(query)
     assert len(res) == 3
 
 def test_query_order_one_to_many_duplicate(client, recwarn):
-    """Note that sorting on a one yo many relation may have surprising
+    """Note that sorting on a one to many relation may have surprising
     effects on the result list.  That is why class Query emits a
     warning.
     You may get duplicates in the result.
@@ -449,7 +559,7 @@ def test_query_order_one_to_many_duplicate(client, recwarn):
     assert set(res) == set(reference)
 
 def test_query_order_one_to_many_missing(client, recwarn):
-    """Note that sorting on a one yo many relation may have surprising
+    """Note that sorting on a one to many relation may have surprising
     effects on the result list.  That is why class Query emits a
     warning.
     You may get misses in the result.
@@ -497,12 +607,17 @@ def test_query_order_suppress_warnings(client, recwarn):
     assert len(res) == 3
 
 def test_query_limit(client):
-    """Add a LIMIT clause to the last example.
+    """Add a LIMIT clause to an earlier example.
     """
     query = Query(client, "Rule", order=['grouping', 'what', 'id'],
                   conditions={"grouping":"IS NOT NULL"})
     query.setLimit( (0,10) )
     print(str(query))
+    assert "Rule" in query.select_clause
+    assert "grouping" in query.join_clause
+    assert "grouping" in query.where_clause
+    assert "what" in query.order_clause
+    assert query.limit_clause is not None
     res = client.search(query)
     assert len(res) == 10
 
@@ -512,13 +627,19 @@ def test_query_limit_placeholder(client):
     query = Query(client, "Rule", order=['grouping', 'what', 'id'],
                   conditions={"grouping":"IS NOT NULL"})
     query.setLimit( ("%d","%d") )
+    chunksize = 45
     print(str(query))
-    print(str(query) % (0,30))
-    res = client.search(str(query) % (0,30))
-    assert len(res) == 30
-    print(str(query) % (30,30))
-    res = client.search(str(query) % (30,30))
-    assert len(res) == 14
+    print(str(query) % (0,chunksize))
+    assert "Rule" in query.select_clause
+    assert "grouping" in query.join_clause
+    assert "grouping" in query.where_clause
+    assert "what" in query.order_clause
+    assert query.limit_clause is not None
+    res = client.search(str(query) % (0,chunksize))
+    assert len(res) == chunksize
+    print(str(query) % (chunksize,chunksize))
+    res = client.search(str(query) % (chunksize,chunksize))
+    assert len(res) == grp_rules - chunksize
 
 def test_query_non_ascii(client):
     """Test if query strings with non-ascii characters work.
@@ -531,10 +652,10 @@ def test_query_non_ascii(client):
     fullName = b'Rudolph Beck-D\xc3\xbclmen'.decode('utf8')
     query = Query(client, "User",
                   conditions={ "fullName": "= '%s'" % fullName })
-    if sys.version_info < (3, 0):
-        print(unicode(query))
-    else:
-        print(str(query))
+    print(str(query))
+    assert "User" in query.select_clause
+    assert query.join_clause is None
+    assert fullName in query.where_clause
     res = client.search(query)
     assert len(res) == 1
 
@@ -556,6 +677,10 @@ def test_query_str(client):
                             "parameters.type.facility"})
     r = repr(query)
     print(str(query))
+    assert "Datafile" in query.select_clause
+    assert "investigation" in query.join_clause
+    assert "id" in query.where_clause
+    assert "datafileFormat" in query.include_clause
     assert repr(query) == r
 
 def test_query_metaattr(client):
@@ -563,6 +688,9 @@ def test_query_metaattr(client):
     """
     query = Query(client, "Datafile", conditions={ "modId": "= 'jdoe'" })
     print(str(query))
+    assert "Datafile" in query.select_clause
+    assert query.join_clause is None
+    assert "modId" in query.where_clause
     res = client.search(query)
     assert len(res) == 0
 
@@ -571,6 +699,11 @@ def test_query_include_1(client):
     """
     query = Query(client, "Investigation", includes="1")
     print(str(query))
+    assert "Investigation" in query.select_clause
+    assert query.join_clause is None
+    assert query.where_clause is None
+    assert "facility" in query.include_clause
+    assert "type" in query.include_clause
     res = client.search(query)
     assert len(res) > 0
     inv = res[0]
@@ -582,13 +715,17 @@ def test_query_include_1(client):
 def test_query_attribute_datafile_name(client):
     """The datafiles names related to a given investigation in natural order.
 
-    Querying attributes rather then entire objects is a new feature
+    Querying attributes rather than entire objects is a new feature
     added in Issue #28.
     """
     query = Query(client, "Datafile", attributes="name", order=True,
                   conditions={ "dataset.investigation.id":
                                "= %d" % investigation.id })
     print(str(query))
+    assert "name" in query.select_clause
+    assert "investigation" in query.join_clause
+    assert "id" in query.where_clause
+    assert query.order_clause is not None
     res = client.search(query)
     assert len(res) == 4
     for n in res:
@@ -605,6 +742,10 @@ def test_query_attribute_datafile_name_list(client):
                   conditions={ "dataset.investigation.id":
                                "= %d" % investigation.id })
     print(str(query))
+    assert "name" in query.select_clause
+    assert "investigation" in query.join_clause
+    assert "id" in query.where_clause
+    assert query.order_clause is not None
     res = client.search(query)
     assert len(res) == 4
     for n in res:
@@ -621,6 +762,10 @@ def test_query_related_obj_attribute(client):
                   conditions={ "dataset.investigation.id":
                                "= %d" % investigation.id })
     print(str(query))
+    assert "name" in query.select_clause
+    assert "datafileFormat" in query.join_clause
+    assert "investigation" in query.join_clause
+    assert "id" in query.where_clause
     res = client.search(query)
     assert len(res) == 4
     for n in res:
@@ -641,6 +786,11 @@ def test_query_mulitple_attributes(client):
     query = Query(client, "Investigation",
                   attributes=("name", "title", "startDate"), order=True)
     print(str(query))
+    assert "name" in query.select_clause
+    assert "title" in query.select_clause
+    assert "startDate" in query.select_clause
+    assert "facility" in query.join_clause
+    assert query.order_clause is not None
     res = client.search(query)
     assert res == results
 
@@ -659,6 +809,10 @@ def test_query_mulitple_attributes_related_obj(client):
                   attributes=("investigation.name", "name"), order=True,
                   conditions={"investigation.startDate":  "< '2011-01-01'"})
     print(str(query))
+    assert "name" in query.select_clause
+    assert "investigation" in query.join_clause
+    assert "startDate" in query.where_clause
+    assert query.order_clause is not None
     res = client.search(query)
     assert res == results
 
@@ -695,37 +849,16 @@ def test_query_mulitple_attributes_distinct(client):
                   conditions={"investigation.name": "= '08100122-EF'"},
                   aggregate="DISTINCT")
     print(str(query))
+    assert "DISTINCT" in query.select_clause
+    assert "role" in query.select_clause
+    assert "investigation" in query.join_clause
+    assert "name" in query.where_clause
     res_dist = client.search(query)
     # The search with DISTINCT yields less items, but if we eliminate
     # duplicates, the result set is the same:
     assert len(res) > len(res_dist)
     assert set(res) == set(res_dist)
 
-@pytest.mark.skipif(Version(pytest.__version__) < "3.9.0",
-                    reason="pytest.deprecated_call() does not work properly")
-def test_query_deprecated_kwarg_attribute(client):
-    """the keyword argument `attribute` to :class:`icat.query.Query`
-    is deprecated since 0.18.0.
-    """
-    # create a reference using the new keyword argument
-    ref_query = Query(client, "Datafile", attributes="name")
-    with pytest.deprecated_call():
-        query = Query(client, "Datafile", attribute="name")
-    assert str(query) == str(ref_query)
-
-@pytest.mark.skipif(Version(pytest.__version__) < "3.9.0",
-                    reason="pytest.deprecated_call() does not work properly")
-def test_query_deprecated_method_setAttribute(client):
-    """:meth:`icat.query.Query.setAttribute` is deprecated since 0.18.0.
-    """
-    # create a reference using the new method
-    ref_query = Query(client, "Datafile")
-    ref_query.setAttributes("name")
-    with pytest.deprecated_call():
-        query = Query(client, "Datafile")
-        query.setAttribute("name")
-    assert str(query) == str(ref_query)
-
 @pytest.mark.dependency(depends=['get_investigation'])
 def test_query_aggregate_distinct_attribute(client):
     """Test DISTINCT on an attribute in the search result.
@@ -743,6 +876,10 @@ def test_query_aggregate_distinct_attribute(client):
     assert sorted(res) == ["NeXus", "NeXus", "other", "other"]
     query.setAggregate("DISTINCT")
     print(str(query))
+    assert "DISTINCT" in query.select_clause
+    assert "name" in query.select_clause
+    assert "datafileFormat" in query.join_clause
+    assert "id" in query.where_clause
     res = client.search(query)
     assert sorted(res) == ["NeXus", "other"]
 
@@ -765,6 +902,10 @@ def test_query_aggregate_distinct_related_obj(client):
         assert isinstance(n, icat.entity.Entity)
     query.setAggregate("DISTINCT")
     print(str(query))
+    assert "DISTINCT" in query.select_clause
+    assert "datafileFormat" in query.select_clause
+    assert "investigation" in query.join_clause
+    assert "id" in query.where_clause
     res = client.search(query)
     assert len(res) == 2
     for n in res:
@@ -782,9 +923,9 @@ def test_query_aggregate_distinct_related_obj(client):
     ("fileSize", "MIN", 394),
     ("fileSize", "SUM", 127125),
     # Note that the number of datafiles is four which is a power of
-    # two.  Therefore we may assume the double representation of an
-    # average of integers is exact, so we may even dare to compare
-    # the double value for equality.
+    # two.  Therefore we may assume the float representation of an
+    # average of integers is exact, so we may even dare to compare the
+    # float value for equality.
     ("fileSize", "AVG", 31781.25),
     ("name", "MAX", "e208341.nxs"),
     ("name", "MIN", "e208339.dat"),
@@ -810,6 +951,12 @@ def test_query_aggregate_misc(client, attribute, aggregate, expected):
                   conditions={ "dataset.investigation.id":
                                "= %d" % investigation.id })
     print(str(query))
+    if ':' not in aggregate:
+        assert aggregate in query.select_clause
+    if attribute is not None and '.' not in attribute:
+        assert attribute in query.select_clause
+    assert "investigation" in query.join_clause
+    assert "id" in query.where_clause
     res = client.search(query)
     assert len(res) == 1
     assert res[0] == expected
@@ -846,3 +993,4 @@ def test_query_copy(client, entity, kwargs):
     query = Query(client, entity, **kwargs)
     clone = query.copy()
     assert str(clone) == str(query)
+    assert entity in clone.select_clause
